package cn.com.qmth.stmms.admin.dto;

import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;

import org.apache.commons.lang.StringUtils;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.mark.model.MarkConfigItem;
import cn.com.qmth.stmms.common.enums.MarkMode;
import cn.com.qmth.stmms.common.enums.QuestionType;
import cn.com.qmth.stmms.common.utils.BigDecimalUtils;

public class SubjectQuestionDTO {

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    private int examId;

    private String subjectCode;

    private List<ExamQuestion> list;

    private double totalScore;

    private boolean objective;

    private Integer trialCount;

    private Map<String, List<ExamQuestion>> paperTypeList;

    private Map<Integer, MarkGroup> groups;

    public SubjectQuestionDTO(int examId, String subjectCode, boolean objective) {
        this.examId = examId;
        this.subjectCode = subjectCode;
        this.list = new LinkedList<ExamQuestion>();
        this.paperTypeList = new HashMap<>();
        this.groups = new HashMap<Integer, MarkGroup>();
        this.totalScore = 0d;
        this.objective = objective;
        this.trialCount = 0;
    }

    public void addQuestion(ExamQuestion question) {
        if (question != null) {
            question.setExamId(examId);
            list.add(question);

            if (objective) {
                String paperType = question.getPaperType();
                if (paperType == null) {
                    paperType = NULL_PAPER_TYPE_PLACEHOLDER;
                }
                List<ExamQuestion> list = paperTypeList.get(paperType);
                if (list == null) {
                    list = new ArrayList<>();
                    paperTypeList.put(paperType, list);
                }
                list.add(question);
            } else {
                MarkGroup group = groups.get(question.getGroupNumber());
                if (group == null) {
                    group = new MarkGroup(examId, subjectCode, question.getGroupNumber(),
                            MarkConfigItem.parse(question.getPicList()), 0d, question.getDoubleRate(),
                            question.getArbitrateThreshold(), question.getScorePolicy(), question.getArbitrateType(),
                            question.getMarkMode(), question.getTrialCount(), false, false, 1, false);
                    group.setImportQuestionList(new LinkedList<>());
                    groups.put(question.getGroupNumber(), group);
                }
                group.getImportQuestionList().add(question);
                this.trialCount = question.getTrialCount() != null && question.getTrialCount() > 0
                        ? question.getTrialCount()
                        : 0;
                // group.setTotalScore(group.getTotalScore() +
                // question.getTotalScore());
                // totalScore += question.getTotalScore();
            }
        }
    }

    public List<ExamQuestion> getQuestionList() {
        return list;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public double getTotalScore() {
        return totalScore;
    }

    public Map<Integer, MarkGroup> getGroups() {
        return groups;
    }

    public Integer getTrialCount() {
        return trialCount;
    }

    public void setTrialCount(Integer trialCount) {
        this.trialCount = trialCount;
    }

    public boolean validate(List<String> error, Map<Integer, String> titleMap) {
        if (list.size() > 0) {
            Set<String> numberSet = new HashSet<String>();
            // Map<Integer, Integer> groupNumberMap = new HashMap<>();
            try {
                // 基本字段合法性校验
                for (ExamQuestion question : list) {
                    if (StringUtils.isBlank(question.getMainTitle())) {
                        error.add("[" + subjectCode + "] 有大题名称为空的记录");
                        return false;
                    }
                    if (question.getMainTitle().length() > 30) {
                        error.add("[" + subjectCode + "] 有大题名称超过30个字的记录");
                        return false;
                    }
                    if (question.getMainNumber() == null || StringUtils.isBlank(question.getSubNumber())) {
                        error.add("[" + subjectCode + "] 有题号为空的记录");
                        return false;
                    }
                    if (!StringUtils.isNumeric(question.getSubNumber())) {
                        error.add("[" + subjectCode + "] 有小题号不是数字的记录");
                        return false;
                    }
                    if (question.getTotalScore() == null) {
                        error.add("[" + subjectCode + "] 有满分为空的记录");
                        return false;
                    }
                    if (countDecimalPlaces(question.getTotalScore()) > 2) {
                        error.add("[" + subjectCode + "] 有满分为小数超2位的记录");
                        return false;
                    }
                    if (!objective && countDecimalPlaces(question.getIntervalScore()) > 2) {
                        error.add("[" + subjectCode + "] 有间隔分为小数超2位的记录");
                        return false;
                    }
                    if (!objective && question.getIntervalScore() <= 0) {
                        error.add("[" + subjectCode + "] 有间隔分有小于等于0的记录");
                        return false;
                    }
                    if (objective && StringUtils.isBlank(question.getAnswer())) {
                        error.add("[" + subjectCode + "] 有答案为空的记录");
                        return false;
                    }
                    if (objective && question.getType() == null) {
                        error.add("[" + subjectCode + "] 大题号" + question.getMainNumber() + "有题型为空的记录");
                        return false;
                    }
                    if (QuestionType.TRUE_OR_FALSE.equals(question.getType())
                            && (!question.getAnswer().contains("A") && !question.getAnswer().contains("B"))) {
                        error.add("[" + subjectCode + "] 大题号" + question.getMainNumber() + "有判断题标答不为A或B的记录");
                        return false;
                    }
                    if (QuestionType.SINGLE.equals(question.getType()) && question.getAnswer().length() > 1) {
                        error.add("[" + subjectCode + "] 大题号" + question.getMainNumber() + "有单选题标答大于一个的记录");
                        return false;
                    }
                    String title = titleMap.get(question.getMainNumber());
                    if (title != null && !title.equals(question.getMainTitle())) {
                        error.add("[" + subjectCode + "] 大题号" + question.getMainNumber() + "有名称不一致的记录");
                        return false;
                    } else {
                        titleMap.put(question.getMainNumber(), question.getMainTitle());
                    }

                    // 不再限制一个大题只能属于一个评卷分组
                    // Integer groupNumber =
                    // groupNumberMap.get(question.getMainNumber());
                    // if (groupNumber != null &&
                    // !groupNumber.equals(question.getGroupNumber())) {
                    // error.add("[" + subjectCode + "] 大题号" +
                    // question.getMainNumber() + "有评卷分组不一致的记录");
                    // return false;
                    // } else {
                    // groupNumberMap.put(question.getMainNumber(),
                    // question.getGroupNumber());
                    // }

                    String number = question.getQuestionNumber();
                    if (objective) {
                        number = StringUtils.trimToEmpty(question.getPaperType()) + " " + number;
                    }
                    if (numberSet.contains(number)) {
                        error.add("[" + subjectCode + "] 题号" + number + "有重复记录");
                        return false;
                    } else {
                        numberSet.add(number);
                    }
                }

                // 客观题时额外校验不同试卷类型的总分需要一致
                if (objective) {
                    totalScore = -1;
                    List<ExamQuestion> questions = null;
                    for (Entry<String, List<ExamQuestion>> entry : paperTypeList.entrySet()) {
                        BigDecimal score = BigDecimal.ZERO;
                        for (ExamQuestion question : entry.getValue()) {
                            score = score.add(BigDecimal.valueOf(question.getTotalScore()));
                        }
                        if (totalScore == -1) {
                            totalScore = score.doubleValue();
                        } else if (totalScore != score.doubleValue()) {
                            error.add("[" + subjectCode + "] 多个试卷类型总分不一致");
                            return false;
                        }
                        if (questions != null) {
                            String message = check(questions, entry.getValue());
                            if (message != null) {
                                error.add("[" + subjectCode + "] 多个试卷类型" + message);
                                return false;
                            }
                        } else {
                            questions = entry.getValue();
                        }
                    }
                }
                return true;
            } finally {
                numberSet.clear();
                titleMap.clear();
                // groupNumberMap.clear();
            }
        } else {
            return false;
        }
    }

    public int countDecimalPlaces(double number) {
        String num = Double.toString(number);
        int decimalPointIndex = num.indexOf('.');
        if (decimalPointIndex == -1) {
            return 0;
        }
        return num.length() - decimalPointIndex - 1;
    }

    private String check(List<ExamQuestion> questions, List<ExamQuestion> value) {
        if (questions.size() != value.size()) {
            return "题目总数不一致";
        }
        Map<Integer, Integer> subCountMap = new HashMap<Integer, Integer>();
        Map<Integer, Double> mainScoreMap = new HashMap<Integer, Double>();
        int maniCount = 0;
        for (ExamQuestion question : questions) {
            if (question.getTotalScore() != 0) {
                Integer subCount = subCountMap.get(question.getMainNumber());
                if (subCount != null) {
                    subCountMap.put(question.getMainNumber(), subCount + 1);
                    Double mainScore = mainScoreMap.get(question.getMainNumber());
                    mainScoreMap.put(question.getMainNumber(),
                            BigDecimalUtils.add(mainScore, question.getTotalScore()));
                } else {
                    subCountMap.put(question.getMainNumber(), 1);
                    mainScoreMap.put(question.getMainNumber(), question.getTotalScore());
                    maniCount++;
                }
            }
        }

        Map<Integer, Integer> subCountMap1 = new HashMap<Integer, Integer>();
        Map<Integer, Double> mainScoreMap1 = new HashMap<Integer, Double>();
        int maniCount1 = 0;
        for (ExamQuestion question : value) {
            if (question.getTotalScore() != 0) {
                Integer subCount = subCountMap1.get(question.getMainNumber());
                if (subCount != null) {
                    subCountMap1.put(question.getMainNumber(), subCount + 1);
                    Double mainScore = mainScoreMap1.get(question.getMainNumber());
                    mainScoreMap1.put(question.getMainNumber(),
                            BigDecimalUtils.add(mainScore, question.getTotalScore()));
                } else {
                    subCountMap1.put(question.getMainNumber(), 1);
                    mainScoreMap1.put(question.getMainNumber(), question.getTotalScore());
                    maniCount1++;
                }
            }
        }
        if (maniCount != maniCount1) {
            return "大题数量不一致";
        }
        for (Integer mainNumber : subCountMap.keySet()) {
            if (!subCountMap.get(mainNumber).equals(subCountMap1.get(mainNumber))) {
                return "小题数量不一致";
            }
            if (!mainScoreMap.get(mainNumber).equals(mainScoreMap1.get(mainNumber))) {
                return "大题总分不一致";
            }
        }
        return null;
    }

    private String errMsg(String err) {
        return errMsg(null, err);
    }

    private String errMsg(Integer groupNumber, String err) {
        if (groupNumber == null) {
            return "[" + subjectCode + "] " + err;
        } else {
            return "[" + subjectCode + "] 分组号" + groupNumber + " " + err;
        }
    }

    public boolean validateGroupNumber(List<String> error) {
        Set<Integer> trialSet = new HashSet<>();
        Map<Integer, Set<Integer>> arbitrateTypeMap = new HashMap<>();
        Map<Integer, Set<Double>> doubleRateMap = new HashMap<>();
        Map<Integer, Set<String>> modeMap = new HashMap<>();
        Map<Integer, Set<Integer>> policyMap = new HashMap<>();
        Map<Integer, Set<Double>> arbitrateThresholdMap = new HashMap<>();

        for (ExamQuestion question : list) {
            if (question.getGroupNumber() == null || question.getGroupNumber() < 0) {
                error.add(errMsg("评卷分组为空"));
                return false;
            }
            trialSet.add(question.getTrialCount());
            if (trialSet.size() > 1) {
                error.add(errMsg("试评数量不一致"));
                return false;
            }
            // 评卷模式一致
            if (StringUtils.isBlank(question.getMarkMode())) {
                error.add(errMsg(question.getGroupNumber(), "评卷模式不能为空"));
                return false;
            }
            if (MarkMode.findByName(question.getMarkMode()) == null) {
                error.add(errMsg(question.getGroupNumber(), "评卷模式填值错误"));
                return false;
            }
            Set<String> modeSet = modeMap.get(question.getGroupNumber());
            if (modeSet == null) {
                modeSet = new HashSet<>();
                modeMap.put(question.getGroupNumber(), modeSet);
            }
            modeSet.add(question.getMarkMode());

            if (modeSet.size() > 1) {
                error.add(errMsg(question.getGroupNumber(), "评卷模式不一致"));
                return false;
            }
            // 双评比例校验
            if (question.getDoubleRate() != null && (question.getDoubleRate() < 0 || question.getDoubleRate() > 1)) {
                error.add(errMsg(question.getGroupNumber(), "双评比例只能是0到1之间的数值（含0、1）"));
                return false;
            }
            Set<Double> doubleRateSet = doubleRateMap.get(question.getGroupNumber());
            if (doubleRateSet == null) {
                doubleRateSet = new HashSet<>();
                doubleRateMap.put(question.getGroupNumber(), doubleRateSet);
            }
            doubleRateSet.add(question.getDoubleRate());
            if (doubleRateSet.size() > 1) {
                error.add(errMsg(question.getGroupNumber(), "双评比例不一致"));
                return false;
            }
            Double doubleRateFirst = null;
            if (!doubleRateSet.isEmpty()) {
                doubleRateFirst = doubleRateSet.iterator().next();
            }
            // 开启双评
            if (doubleRateFirst != null && doubleRateFirst > 0) {
                if (question.getArbitrateType() == null) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁方式为空"));
                    return false;
                }
                if (question.getArbitrateType() != 0 && question.getArbitrateType() != 1) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁方式值只能是0或1"));
                    return false;
                }
                // 仲裁方式是否一致
                Set<Integer> arbitrateTypeSet = arbitrateTypeMap.get(question.getGroupNumber());
                if (arbitrateTypeSet == null) {
                    arbitrateTypeSet = new HashSet<>();
                    arbitrateTypeMap.put(question.getGroupNumber(), arbitrateTypeSet);
                }
                arbitrateTypeSet.add(question.getArbitrateType());
                if (arbitrateTypeSet.size() > 1) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁方式不一致"));
                    return false;
                }

                if (question.getArbitrateThreshold() == null) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁阈值为空"));
                    return false;
                }

                if (question.getArbitrateThreshold() != null && question.getArbitrateThreshold() < 0) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁阈值不能为负数"));
                    return false;
                }

                Set<Double> arbitrateThresholdSet = arbitrateThresholdMap.get(question.getGroupNumber());
                if (arbitrateThresholdSet == null) {
                    arbitrateThresholdSet = new HashSet<>();
                    arbitrateThresholdMap.put(question.getGroupNumber(), arbitrateThresholdSet);
                }
                arbitrateThresholdSet.add(question.getArbitrateThreshold());

                Integer arbitrateTypeFirst = null;
                if (!arbitrateTypeSet.isEmpty()) {
                    arbitrateTypeFirst = arbitrateTypeSet.iterator().next();
                }
                if (arbitrateTypeFirst != null && !arbitrateTypeFirst.equals(1) && arbitrateThresholdSet.size() > 1) {
                    error.add(errMsg(question.getGroupNumber(), "仲裁阈值不一致"));
                    return false;
                }

                // 合分策略一致
                if (question.getScorePolicy() == null) {
                    error.add(errMsg(question.getGroupNumber(), "合分策略为空"));
                    return false;
                }
                if (question.getScorePolicy() != 1 && question.getScorePolicy() != 2
                        && question.getScorePolicy() != 3) {
                    error.add(errMsg(question.getGroupNumber(), "合分策略值只能是1、2、3"));
                    return false;
                }
                Set<Integer> policySet = policyMap.get(question.getGroupNumber());
                if (policySet == null) {
                    policySet = new HashSet<>();
                    policyMap.put(question.getGroupNumber(), policySet);
                }
                policySet.add(question.getScorePolicy());
                if (policySet.size() > 1) {
                    error.add(errMsg(question.getGroupNumber(), "合分策略不一致"));
                    return false;
                }
            }
        }
        return true;
    }

}
